@@ -146,6 +146,7 @@ typedef struct daemon_t daemon_t;
 #include <bus/listeners/sys_logger.h>
 #include <sa/ike_sa_manager.h>
 #include <sa/trap_manager.h>
+#include <sa/shunt_manager.h>
 #include <config/backend_manager.h>
 #include <sa/authenticators/eap/eap_manager.h>
 #include <sa/authenticators/eap/sim_manager.h>
@@ -193,6 +194,11 @@ struct daemon_t {
 	 */
 	trap_manager_t *traps;
 
+	/**
+	 * Manager for shunt PASS|DROP policies
+	 */
+	shunt_manager_t *shunts;
+
 	/**
 	 * Manager for the different configuration backends.
 	 */
