@@ -86,7 +86,7 @@ METHOD(kernel_ipsec_t, add_policy, status_t,
 	   private_load_tester_ipsec_t *this, host_t *src, host_t *dst,
 	   traffic_selector_t *src_ts, traffic_selector_t *dst_ts,
 	   policy_dir_t direction, policy_type_t type, ipsec_sa_cfg_t *sa,
-	   mark_t mark, bool routed)
+	   mark_t mark, policy_priority_t priority)
 {
 	return SUCCESS;
 }
@@ -96,14 +96,14 @@ METHOD(kernel_ipsec_t, query_policy, status_t,
 	   traffic_selector_t *dst_ts, policy_dir_t direction, mark_t mark,
 	   u_int32_t *use_time)
 {
-	*use_time = time_monotonic(NULL);
+	*use_time = 1;
 	return SUCCESS;
 }
 
 METHOD(kernel_ipsec_t, del_policy, status_t,
 	   private_load_tester_ipsec_t *this, traffic_selector_t *src_ts,
-	   traffic_selector_t *dst_ts, policy_dir_t direction, mark_t mark,
-	   bool unrouted)
+	   traffic_selector_t *dst_ts, policy_dir_t direction, u_int32_t reqid,
+	   mark_t mark, policy_priority_t priority)
 {
 	return SUCCESS;
 }
