@@ -76,8 +76,7 @@ static void write_fifo(private_uci_control_t *this, char *format, ...)
  */
 static void status(private_uci_control_t *this, char *name)
 {
-	enumerator_t *configs, *sas;
-	iterator_t *children;
+	enumerator_t *configs, *sas, *children;
 	ike_sa_t *ike_sa;
 	child_sa_t *child_sa;
 	peer_cfg_t *peer_cfg;
@@ -92,7 +91,8 @@ static void status(private_uci_control_t *this, char *name)
 		{
 			continue;
 		}
-		sas = charon->controller->create_ike_sa_enumerator(charon->controller);
+		sas = charon->controller->create_ike_sa_enumerator(
+													charon->controller, TRUE);
 		while (sas->enumerate(sas, &ike_sa))
 		{
 			if (!streq(ike_sa->get_name(ike_sa), peer_cfg->get_name(peer_cfg)))
@@ -110,8 +110,8 @@ static void status(private_uci_control_t *this, char *name)
 			fprintf(out, "%-8s %-20D %-16H ", ike_sa->get_name(ike_sa),
 				ike_sa->get_other_id(ike_sa), ike_sa->get_other_host(ike_sa));
 
-			children = ike_sa->create_child_sa_iterator(ike_sa);
-			while (children->iterate(children, (void**)&child_sa))
+			children = ike_sa->create_child_sa_enumerator(ike_sa);
+			while (children->enumerate(children, (void**)&child_sa))
 			{
 				fprintf(out, "%#R",
 						child_sa->get_traffic_selectors(child_sa, FALSE));
@@ -174,7 +174,8 @@ static void terminate(private_uci_control_t *this, char *name)
 	ike_sa_t *ike_sa;
 	u_int id;
 
-	enumerator = charon->controller->create_ike_sa_enumerator(charon->controller);
+	enumerator = charon->controller->create_ike_sa_enumerator(
+													charon->controller, TRUE);
 	while (enumerator->enumerate(enumerator, &ike_sa))
 	{
 		if (streq(name, ike_sa->get_name(ike_sa)))
@@ -292,8 +293,8 @@ uci_control_t *uci_control_create()
 	}
 	else
 	{
-		this->job = callback_job_create((callback_job_cb_t)receive,
-										this, NULL, NULL);
+		this->job = callback_job_create_with_prio((callback_job_cb_t)receive,
+										this, NULL, NULL, JOB_PRIO_CRITICAL);
 		lib->processor->queue_job(lib->processor, (job_t*)this->job);
 	}
 	return &this->public;
