@@ -78,24 +78,6 @@ struct private_ike_mobike_t {
 	bool addresses_updated;
 };
 
-/**
- * flush the IKE_SAs list of additional addresses
- */
-static void flush_additional_addresses(private_ike_mobike_t *this)
-{
-	iterator_t *iterator;
-	host_t *host;
-
-	iterator = this->ike_sa->create_additional_address_iterator(this->ike_sa);
-	while (iterator->iterate(iterator, (void**)&host))
-	{
-		iterator->remove(iterator);
-		host->destroy(host);
-	}
-	iterator->destroy(iterator);
-}
-
-
 /**
  * read notifys from message and evaluate them
  */
@@ -152,7 +134,7 @@ static void process_payloads(private_ike_mobike_t *this, message_t *message)
 			{
 				if (first)
 				{	/* an ADDITIONAL_*_ADDRESS means replace, so flush once */
-					flush_additional_addresses(this);
+					this->ike_sa->remove_additional_addresses(this->ike_sa);
 					first = FALSE;
 				}
 				data = notify->get_notification_data(notify);
@@ -169,7 +151,7 @@ static void process_payloads(private_ike_mobike_t *this, message_t *message)
 			}
 			case NO_ADDITIONAL_ADDRESSES:
 			{
-				flush_additional_addresses(this);
+				this->ike_sa->remove_additional_addresses(this->ike_sa);
 				this->addresses_updated = TRUE;
 				break;
 			}
@@ -256,11 +238,11 @@ static void build_cookie(private_ike_mobike_t *this, message_t *message)
  */
 static void update_children(private_ike_mobike_t *this)
 {
-	iterator_t *iterator;
+	enumerator_t *enumerator;
 	child_sa_t *child_sa;
 
-	iterator = this->ike_sa->create_child_sa_iterator(this->ike_sa);
-	while (iterator->iterate(iterator, (void**)&child_sa))
+	enumerator = this->ike_sa->create_child_sa_enumerator(this->ike_sa);
+	while (enumerator->enumerate(enumerator, (void**)&child_sa))
 	{
 		if (child_sa->update(child_sa,
 				this->ike_sa->get_my_host(this->ike_sa),
@@ -273,7 +255,7 @@ static void update_children(private_ike_mobike_t *this)
 					child_sa->get_spi(child_sa, TRUE));
 		}
 	}
-	iterator->destroy(iterator);
+	enumerator->destroy(enumerator);
 }
 
 /**
@@ -296,7 +278,7 @@ METHOD(ike_mobike_t, transmit, void,
 	   private_ike_mobike_t *this, packet_t *packet)
 {
 	host_t *me, *other, *me_old, *other_old;
-	iterator_t *iterator;
+	enumerator_t *enumerator;
 	ike_cfg_t *ike_cfg;
 	packet_t *copy;
 
@@ -320,8 +302,8 @@ METHOD(ike_mobike_t, transmit, void,
 		charon->sender->send(charon->sender, copy);
 	}
 
-	iterator = this->ike_sa->create_additional_address_iterator(this->ike_sa);
-	while (iterator->iterate(iterator, (void**)&other))
+	enumerator = this->ike_sa->create_additional_address_enumerator(this->ike_sa);
+	while (enumerator->enumerate(enumerator, (void**)&other))
 	{
 		me = hydra->kernel_interface->get_source_addr(
 										hydra->kernel_interface, other, NULL);
@@ -343,7 +325,7 @@ METHOD(ike_mobike_t, transmit, void,
 			charon->sender->send(charon->sender, copy);
 		}
 	}
-	iterator->destroy(iterator);
+	enumerator->destroy(enumerator);
 }
 
 METHOD(task_t, build_i, status_t,
