@@ -21,12 +21,14 @@
 #ifndef IMC_MANAGER_H_
 #define IMC_MANAGER_H_
 
+typedef struct imc_manager_t imc_manager_t;
+
+#ifdef USE_TNC
+
 #include "imc.h"
 
 #include <library.h>
 
-typedef struct imc_manager_t imc_manager_t;
-
 /**
  * The IMC manager controls all IMC instances.
  */
@@ -121,4 +123,6 @@ struct imc_manager_t {
 	void (*destroy)(imc_manager_t *this);
 };
 
+#endif /* USE_TNC */
+
 #endif /** IMC_MANAGER_H_ @}*/
