@@ -24,17 +24,11 @@
 #ifndef TNCCS_H_
 #define TNCCS_H_
 
-#include <tnc/tncif.h>
-#include <tnc/tncifimc.h>
-#include <tnc/tncifimv.h>
-#include <library.h>
+#include <tncif.h>
+#include <tncifimc.h>
+#include <tncifimv.h>
 
-#define IETF_VENDOR_ID			0x000000	/*        0 */
-#define MICROSOFT_VENDOR_ID 	0x000137	/*      311 */
-#define OSC_VENDOR_ID			0x002358	/*     9048 */
-#define FHH_VENDOR_ID			0x0080ab	/*    32939 */
-#define ITA_VENDOR_ID			0x00902a	/*    36906 */
-#define RESERVED_VENDOR_ID		0xffffff	/* 16777215 */
+#include <library.h>
 
 typedef enum tnccs_type_t tnccs_type_t;
 
