@@ -28,6 +28,8 @@ ENUM(debug_names, DBG_DMN, DBG_LIB,
 	"NET",
 	"ENC",
 	"TNC",
+	"IMC",
+	"IMV",
 	"TLS",
 	"LIB",
 );
@@ -43,6 +45,8 @@ ENUM(debug_lower_names, DBG_DMN, DBG_LIB,
 	"net",
 	"enc",
 	"tnc",
+	"imc",
+	"imv",
 	"tls",
 	"lib",
 );
