@@ -231,7 +231,18 @@ static int show_status(stroke_keyword_t kw, char *connection)
 {
 	stroke_msg_t msg;
 
-	msg.type = (kw == STROKE_STATUS)? STR_STATUS:STR_STATUS_ALL;
+	switch (kw)
+	{
+		case STROKE_STATUSALL:
+			msg.type = STR_STATUS_ALL;
+			break;
+		case STROKE_STATUSALL_NOBLK:
+			msg.type = STR_STATUS_ALL_NOBLK;
+			break;
+		default:
+			msg.type = STR_STATUS;
+			break;
+	}
 	msg.length = offsetof(stroke_msg_t, buffer);
 	msg.status.name = push_string(&msg, connection);
 	return send_stroke_msg(&msg);
@@ -326,6 +337,15 @@ static int leases(stroke_keyword_t kw, char *pool, char *address)
 	return send_stroke_msg(&msg);
 }
 
+static int memusage()
+{
+	stroke_msg_t msg;
+
+	msg.type = STR_MEMUSAGE;
+	msg.length = offsetof(stroke_msg_t, buffer);
+	return send_stroke_msg(&msg);
+}
+
 static int set_loglevel(char *type, u_int level)
 {
 	stroke_msg_t msg;
@@ -373,6 +393,10 @@ static void exit_usage(char *error)
 	printf("           LEVEL is -1|0|1|2|3|4\n");
 	printf("  Show connection status:\n");
 	printf("    stroke status\n");
+	printf("  Show extended status information:\n");
+	printf("    stroke statusall\n");
+	printf("  Show extended status information without blocking:\n");
+	printf("    stroke statusallnb\n");
 	printf("  Show list of authority and attribute certificates:\n");
 	printf("    stroke listcacerts|listocspcerts|listaacerts|listacerts\n");
 	printf("  Show list of end entity certificates, ca info records  and crls:\n");
@@ -393,6 +417,8 @@ static void exit_usage(char *error)
 	printf("    stroke purgeike\n");
 	printf("  Export credentials to the console:\n");
 	printf("    stroke exportx509 DN\n");
+	printf("  Show current memory usage:\n");
+	printf("    stroke memusage\n");
 	printf("  Show leases of a pool:\n");
 	printf("    stroke leases [POOL [ADDRESS]]\n");
 	exit_error(error);
@@ -489,6 +515,7 @@ int main(int argc, char *argv[])
 			break;
 		case STROKE_STATUS:
 		case STROKE_STATUSALL:
+		case STROKE_STATUSALL_NOBLK:
 			res = show_status(token->kw, argc > 2 ? argv[2] : NULL);
 			break;
 		case STROKE_LIST_PUBKEYS:
@@ -530,6 +557,9 @@ int main(int argc, char *argv[])
 			res = leases(token->kw, argc > 2 ? argv[2] : NULL,
 						 argc > 3 ? argv[3] : NULL);
 			break;
+		case STROKE_MEMUSAGE:
+			res = memusage();
+			break;
 		default:
 			exit_usage(NULL);
 	}
